package ru.ulstu.report.service;

import org.springframework.stereotype.Service;
import ru.ulstu.aspirant.model.Aspirant;
import ru.ulstu.aspirant.service.AspirantService;
import ru.ulstu.report.model.Report;
import ru.ulstu.report.model.ReportPeriod;
import ru.ulstu.report.model.ReportValue;
import ru.ulstu.report.model.dto.ReportDto;
import ru.ulstu.report.repository.ReportRepository;
import ru.ulstu.user.UserService;

import java.util.List;

@Service
public class ReportService {
    private final ReportRepository reportRepository;
    private final UserService userService;
    private final AspirantService aspirantService;
    private final ReportPeriodService reportPeriodService;

    public ReportService(ReportRepository reportRepository,
                         UserService userService,
                         AspirantService aspirantService, ReportPeriodService reportPeriodService) {
        this.reportRepository = reportRepository;
        this.userService = userService;
        this.aspirantService = aspirantService;
        this.reportPeriodService = reportPeriodService;
    }

    public List<Report> getReports(ReportPeriod reportPeriod) {
        return reportRepository.findAllByReportPeriod(reportPeriod);
    }

    public boolean canCreateReport(ReportPeriod reportPeriod) {
        Aspirant currentAspirant = aspirantService.getAspirantByUser(userService.getCurrentUser());
        return reportRepository.findByReportPeriodAndAspirant(reportPeriod, currentAspirant) == null;
    }

    public Report saveReport(ReportDto reportDto) {
        Report report;
        if (reportDto.getId() == null || reportDto.getId() == 0) {
            report = new Report();
        } else {
            report = getReportById(reportDto.getId());
        }
        if (reportDto.getReportPeriodId() != null) {
            report.setReportPeriod(reportPeriodService.getById(reportDto.getReportPeriodId()));
        }
        report.setAspirant(aspirantService.getAspirantByUser(userService.getCurrentUser()));
        report.setCreateDate(reportDto.getCreateDate());
        report.setStatus(reportDto.getStatus());
        return reportRepository.save(report);
    }

    public Report getReportById(Integer reportId) {
        return reportRepository
                .findById(reportId)
                .orElseThrow(() -> new RuntimeException("Report not found by id"));
    }

    public void deleteReport(Integer id) {
        reportRepository.deleteById(id);
    }

    public void addReportValue(Integer reportId, ReportValue reportValue) {
        Report report = getReportById(reportId);
        report.getValues().remove(reportValue);
        report.getValues().add(reportValue);
        reportRepository.save(report);
    }
}
